

from ..Utilities.SingletonBase import Singleton
from .MaterialsCreator import MaterialsCreator
from ..Utilities.AssetData import *

import hou



class RedshiftMaterialFactory:
    __metaclass__ = Singleton
    def __init_(self):
        pass
    
    def createMaterial(self, assetData, importParams, importOptions):
        gammaCorrected = ["albedo", "diffuse", "specular", "translucency"]
        
        self.redshiftMaterialSettings = {
            "mapConnections" :{
                "albedo" : {
                    "input" : "diffuse_color",
                    "output" : "outColor"
                },
                "diffuse" : {
                    "input" : "diffuse_color",
                    "output" : "outColor"
                },

                "roughness" : {
                    "input" : "refl_roughness",
                    "output" : "outColor"
                },

                "specular" : {
                    "input" : "refl_color",
                    "output" : "outColor"
                },
                "normal" : {
                    "input" : "bump_input",
                    "output" : "outDisplacementVector"
                },
                "bump" : {
                    "input" : "Bump Map",
                    "output" : "out"
                },
                "opacity" : {
                    "input" : "opacity_color",
                    "output" : "outColor"
                },
                "displacement" :{
                    "input" : "Displacement",
                    "output" : "out"
                },
                "metalness" : {
                    "input" : "refl_metalness",
                    "output" : "outR"
                },
                "cavity" : {
                    "input" : "overall_color",
                    "output" : "outColor"
                }
            }

        }
        materialPath = importParams["materialPath"]
        # redshiftMatBuilder = hou.node(materialPath).createNode("redshift_vopnet", importParams["assetName"])
        materialContainer = hou.node(materialPath)        
        shaderNode = materialContainer.createNode("redshift::Material")
        shaderNode.parm("refl_brdf").set("1")
        # shaderNode.parm("refl_fresnel_mode").set("2")
        materialNode = None
        bumpPriority = False
        for textureData in assetData["components"]:            
            
            
            
            
            if textureData["type"] != "normal":
                textureNode = materialContainer.createNode("redshift::TextureSampler", textureData["type"])
                textureNode.parm("tex0").set(textureData["path"].replace("\\", "/"))

            if textureData["type"] not in self.redshiftMaterialSettings["mapConnections"].keys(): continue
            textureParams =  self.redshiftMaterialSettings["mapConnections"][textureData["type"]]

            # if textureData["type"] in gammaEnabled:
            #     textureNode.parm("tex0_gammaoverride").set(1)

            if textureData["type"] == "roughness":
                rampNode = materialContainer.createNode("redshift::RSRamp")                
                # channelSplitter = materialContainer.createNode("redshift::RSColorSplitter")
                # channelSplitter.setNamedInput("input", textureNode, "outColor" )
                rampNode.setNamedInput("input", textureNode, "outColor" )
                # rampNode.setNamedInput("input", channelSplitter, "outR")
                shaderNode.setNamedInput(textureParams["input"], rampNode, textureParams["output"])
                

            elif textureData["type"] == "metalness":
                channelSplitter2 = materialContainer.createNode("redshift::RSColorSplitter")
                shaderNode.setNamedInput(textureParams["input"], channelSplitter2, textureParams["output"])
                channelSplitter2.setNamedInput("input", textureNode, "outColor" )

            elif textureData["type"] == "normal":
                normalNode = materialContainer.createNode("redshift::NormalMap")
                normalNode.parm("tex0").set(textureData["path"].replace("\\", "/"))
                normalNode.parm("flipY").set(1)
                # if materialNode is None:
                #     materialNode = materialContainer.createNode("redshift_material", importParams["assetName"])
                # if bumpPriority == False:                    
                shaderNode.setNamedInput(textureParams["input"], normalNode, textureParams["output"])   

            elif textureData["type"] == "bump":
                bumpNode = materialContainer.createNode("redshift::BumpMap")
                # bumpNode.parm("inputType").set("1")
                if materialNode is None:
                    materialNode = materialContainer.createNode("redshift_material", importParams["assetName"])
                materialNode.setNamedInput(textureParams["input"], bumpNode, textureParams["output"])   
                bumpNode.setNamedInput("input", textureNode, "outColor")
                bumpPriority = True

            elif textureData["type"] == "displacement":
                exrPath = getExrDisplacement(textureData["path"])                
                textureNode.parm("tex0").set(exrPath.replace("\\", "/"))

                dispNode = materialContainer.createNode("redshift::Displacement")
                if materialNode is None:
                    materialNode = materialContainer.createNode("redshift_material", importParams["assetName"])

                materialNode.setNamedInput(textureParams["input"], dispNode, textureParams["output"])
                dispNode.setNamedInput("texMap", textureNode, "outColor")

            elif textureData["type"] == "albedo" or textureData["type"] == "diffuse":
                ccNode = materialContainer.createNode("redshift::RSColorCorrection")
                ccNode.setNamedInput("input", textureNode, "outColor")
                shaderNode.setNamedInput("diffuse_color", ccNode, "outColor")

            # elif textureData["type"] == "specular":
            #     shaderNode.setNamedInput(textureParams["input"], textureNode, textureParams["output"])
            #     shaderNode.setNamedInput("refl_color", textureNode, textureParams["output"])                

            else:                
                shaderNode.setNamedInput(textureParams["input"], textureNode, textureParams["output"])
                

        if materialNode is not None:
            materialNode.setNamedInput("Surface", shaderNode, "outColor")
            
            return materialNode

        return shaderNode


class RedshiftTriplanarFactory:
    __metaclass__ = Singleton
    def __init__(self):
        pass

    def createMaterial(self, assetData, importParams, importOptions):
        self.triplanarSettings = {
            "mapConnections" : {
                "albedo" : "diffuse_color",
                "diffuse" : "diffuse_color",
                "roughness" : "refl_roughness",
                "specular" : "refl_reflectivity",
                "normal" : "bump_input",
                "bump" : "Bump Map",
                "opacity" : "opacity_color",
                "displacement" : "Displacement",
                "metalness" : "refl_metalness"
                
            }

        }

        materialContainer = hou.node(importParams["materialPath"])        
        shaderNode = materialContainer.createNode("redshift::Material")
        shaderNode.parm("refl_brdf").set("1")
        # shaderNode.parm("refl_fresnel_mode").set("2")
        materialNode = materialContainer.createNode("redshift_material", importParams["assetName"])
        materialNode.setNamedInput("Surface", shaderNode, "outColor")
        triplanarNode = materialContainer.createNode("quixel_redshift_triplanar::1.0")
        for textureData in assetData["components"]:            
            if triplanarNode.parm(textureData["type"]) is not None:
                triplanarNode.parm(textureData["type"]).set(textureData["path"].replace("\\", "/"))
                if textureData["type"] not in self.triplanarSettings["mapConnections"].keys(): continue

                if textureData["type"] == "displacement":
                    dispNode = materialContainer.createNode("redshift::Displacement")
                    materialNode.setNamedInput("Displacement", dispNode, "out")
                    dispNode.setNamedInput("texMap", triplanarNode, "displacement")

                elif textureData["type"] == "bump" :
                    bumpNode = materialContainer.createNode("redshift::BumpMap")
                    materialNode.setNamedInput("Bump Map", bumpNode, "out")   
                    bumpNode.setNamedInput("input", triplanarNode, "bump")
                else:
                    shaderNode.setNamedInput(self.triplanarSettings["mapConnections"][textureData["type"]], triplanarNode, textureData["type"])

        return materialNode

       



def registerMaterials():
    
    # materialTypes = ["Principled", "Triplanar"]
    redshiftTextures = RedshiftMaterialFactory()
    redshiftTriplanar = RedshiftTriplanarFactory()
    materialsCreator = MaterialsCreator()
    materialsCreator.registerMaterial("Redshift", "Redshift Material", redshiftTextures.createMaterial)
    materialsCreator.registerMaterial("Redshift", "Redshift Triplanar", redshiftTriplanar.createMaterial)
    

registerMaterials()
